# frozen_string_literal: true

class OrdersController < ApplicationController
  model_data_store(Order)

  def create
    data = model_params

    new_order_params = data.dup

    @model = Order.new(new_order_params)
    @model.submitted_by = current_user[:name]
    @model.origin = 'RXSITE'

    if @model.validate
      # only generate a po number if it's blank or it's not the va
      order_number = if data['po_number'].present? || RxuVersion.instance.va?
                       Order.submit_order!(data)
                     else
                       RxModule::SerialNumber.for(Order, 'po_number').try_dispense do |po_number|
                         data['po_number'] = po_number
                         @model.po_number = data['po_number']
                         Order.submit_order!(data)
                       end
                     end

      @model.order_number = order_number
      @model.sync_with_rxuniverse!

      # TODO: Update this when the stock order feature is completed
      # https://dev.azure.com/ITRxSolutions/Rx-Universe/_workitems/edit/16575
      if data["job_type"] == "F"
        @model.date_ordered = data["date_ordered"]
        @model._fstatus = data["_fstatus"]
        @model.frame_sku = data["frame_sku"]
        frame_data = {"raw_data" => JSON.parse(@model.raw_data), "frame_sku" => data["frame_sku"]}
        frame_data = Order.set_frame_information(frame_data)
        frame_data["raw_data"] = JSON.generate(frame_data["raw_data"])
        @model.assign_attributes(frame_data)
      end

      @model.save
      render json: @model, status: :created, location: @model
    else
      render json: @model.errors.full_messages, status: :unprocessable_entity
    end
  end

  def duplicate?
    result = Order.duplicate_order?(params[:po_number], params[:customer_id])
    render json: { result: result }
  end

  def order_columns
    columns = Order.column_names.excluding(%w[id created_at updated_at raw_data _fsku cust_job_id style_material_id style_type_id style_id shipping_method_id]).including(%w[style_type_name style_name style_material_name shipping_method_code])
    render json: columns
  end

  private

  # Only allow a list of trusted parameters through.
  def model_params
    permitted = [
      :_fmupc, :_fsku, :_fstatus, :customer_city, :customer_name, :customer_num, :date_completed, :date_ordered, :date_received, :fcol, :fram, :fupc, :job_type, :lens_color, :lens_treat, :order_number, :po_number, :raw_data, :release_number, :ship_to_address1, :ship_to_address2, :ship_to_address3, :ship_to_address4, :ship_to_code, :ship_to_name, :ship_to_num, :tray, :cust_job_id, :trace_id,
      :style_type_id, :style_id, :style_material_id, :customer_id, :shipping_method_id, :frame_sku, :hold_at_lab, :comments, :submitted_by, :addon_ids, :raw_data
    ]
    data = JSON.parse(params["order"], symbolize_names: true).keep_if { |k, _| permitted.include?(k) }
    data[:trace_file] = params[:trace_file]

    data.with_indifferent_access
  end

  def fix_params(param)
    param[:id] = 'styles.style' if param[:id] == 'style_name'
    param[:id] = 'style_types.style_type' if param[:id] == 'style_type_name'
    param[:id] = 'style_materials.material_description' if param[:id] == 'style_material_name'
    param[:id] = 'shipping_methods.code' if param[:id] == 'shipping_method_code'
    param
  end

  def before_sort(sort)
    fix_params(sort)
  end

  def before_filter(filter)
    if filter[:id] == 'customer_id'
      filter[:strict_match] = true
      filter[:value] = filter[:value].split('|')
    end

    filter[:strict_match] = true if filter[:id] == 'hold_at_lab'
    filter[:strict_match] = true if filter[:id] == 'is_valid'
    filter[:strict_match] = true if filter[:id] == 'is_shipped'
    filter[:strict_match] = true if filter[:id] == 'job_track_station'

    fix_params(filter)
  end

  def before_set_targets
    @models = if RxModule::Authorization.current_user_can?('config', 'user_config_admin')
                @models
              else
                config = Config.find_by(user_id: RxModule::Authorization.current_user[:id])
                @models = @models.where(customer_id: config.config_customers.map(&:customer_id))
              end
    @models.eager_load(:style, :style_type, :style_material, :customer, :shipping_method)
  end
end
