import * as React from 'react';
import { observer } from 'mobx-react-lite';
import {ButtonField, ModelForm, PaddedPaper, LinkButton} from "@rx-suite/ui";
import {useParams} from "react-router-dom";
import {Container, LinearProgress, Grid, Typography, makeStyles, Box} from "@material-ui/core";
import {Transport} from "@rx-suite/data";
import MenuOpen from "@material-ui/icons/MenuOpen";
import {t, Trans} from "@lingui/macro";
import {LabMaintenanceStore} from "../../stores/LabMaintenanceStore";
import {ConfirmationTemplateSelectField} from "../../components/confirmation/ConfirmationTemplateSelectField";
import {ConfirmationSaveButton} from "../../components/confirmation/ConfirmationSaveButton";
import {OrderStore} from "../../stores/OrderStore";
import {Refresh} from "@material-ui/icons";
import {ConfirmationError} from "../../components/confirmation/ConfirmationError";

const useClasses = makeStyles(theme => ({
    linkButton: {
        margin: theme.spacing(1)
    }
}))


// Though not used right now, this method will be needed when we add a the "view workticket" button shortly, so not deleting it yet
// export const downloadTicket = async (template: string, id: string, output: string) => {
//     const resp = await Transport.fetch(`/api/rx_module/communication`, {
//         method: "POST",
//         headers: {
//             'Content-Type': 'application/json'
//         },
//         body: JSON.stringify({
//             module_id: "rx_plore",
//             endpoint: `/api/tickets/${template}/svg/${id}.json?mode=API&output=${output}`,
//             method: "get",
//             parameters: {}
//         })
//     });
//     return resp;
// }

export const downloadTicket = async (id: string, output: string) => {
    return Transport.fetch(`/api/orders/${id}/workticket?output=${output}`);
}

export const ConfirmationPage = observer(() => {
    const { id } = useParams();

    const labConfig = LabMaintenanceStore.selectedModel;
    const order = OrderStore.get(id);
    const [ loading, setLoading ] = React.useState(false);
    const [ response, setResponse] = React.useState("");
    const [ error, setError ] = React.useState("");
    const classes = useClasses();

    const handleRefresh = React.useCallback(() => {
        if(!labConfig?.confirmation_template || !order) {
            return;
        }
        const tmp = async () => {
            setLoading(true);

            const resp= await downloadTicket(order.getId(), "svg")

            const svg = await resp.text();

            try {
                const json = JSON.parse(svg);
                setError(json.error);
                setLoading(false);
                return;
            } catch(e) {
                // it's not json, so it's fine
            }

            setError("");

            const domParser = new DOMParser();
            const svgDocument = domParser.parseFromString(svg, "image/svg+xml");
            const svgElement = svgDocument.querySelector("svg");

            if(svgElement) {
                const width = svgElement.getAttribute("width");
                const height = svgElement.getAttribute("height");

                svgElement.setAttribute("viewBox", `0 0 ${width} ${height}`);
                svgElement.setAttribute("width", "100%");
                svgElement.setAttribute("height", "100%");

                setResponse(svgElement.outerHTML);
            }

            setLoading(false);
        }
        tmp();
    }, [order, labConfig?.confirmation_template]);

    React.useEffect(() => {
        handleRefresh();
    }, [handleRefresh]);

    let child =  <LinearProgress />;

    if(!order) return <LinearProgress />

    if(loading || !labConfig) {
        child = (
            <>
                <Typography variant={"h6"} align={"center"}>
                    <Trans>Generating...</Trans>
                </Typography>
                <LinearProgress/>
            </>

        )
    } else if(error) {
        child = <ConfirmationError order={order} handleRefresh={handleRefresh} />
    } else if(!labConfig.confirmation_template) {
        child = (
            <>
                <Typography variant={"h6"}>
                    <Trans>
                        No confirmation template selected, please select a template:
                    </Trans>
                </Typography>
                <ModelForm model={labConfig} suppressWrapper suppressImportExport>
                    <ConfirmationTemplateSelectField />
                </ModelForm>
            </>

        )
    } else {
        child = <div dangerouslySetInnerHTML={{ __html: response }} />
    }

   return(
       <Container>
           <Grid container spacing={2}>
               <Grid item xs={12}>
                   <PaddedPaper>
                       <Box display={"flex"} justifyContent={"space-between"}>
                           <div>
                               <LinkButton to={`/orders/${id}/read`} color={"primary"} className={classes.linkButton} noGutter><MenuOpen/>{t`Order`}</LinkButton>
                               <LinkButton to={`/orders/create`} color={"secondary"} className={classes.linkButton} noGutter>{t`New Order`}</LinkButton>
                           </div>
                           <div>
                               <ButtonField loading={loading} text={<><Refresh /> <Trans>Refresh</Trans></>} onClick={handleRefresh}/>
                               <ConfirmationSaveButton mode={"save"} order={order} disabled={loading || !!error} />
                               <ConfirmationSaveButton mode={"print"} order={order} disabled={loading || !!error}/>
                           </div>
                       </Box>
                   </PaddedPaper>
               </Grid>
               <Grid item xs={12}>
                   <PaddedPaper>
                       {child}
                   </PaddedPaper>
               </Grid>
           </Grid>
       </Container>

   )
});