import {
    HintButtonProps,
    InputField,
    SelectField, useFlag,
    useModelContext,
    useReadonlyContext
} from '@rx-suite/ui';
import {observer} from 'mobx-react-lite';
import * as React from 'react';
import {OrderFieldProps} from "../../../stores/_FieldConstants";
import {getLeftName} from "../../../hooks/useOrderEntryStore";
import {OrderModel} from "../../../stores/OrderStore";

const excludedColumns = ['SPH', 'CYL', 'AX'];
const hintText: { [key: string]: HintButtonProps } = {
    'raw_data.rtPRVA1': {
        title: 'Right Prism Axis 1',
        subtitle: 'Prism axis accepts the following Directional or Diopter values as valid input',
        text: 'Directional: Up, Down, In, Out\n' +
            'Diopter range: 0-359 (dpt)'
    },
    'raw_data.ltPRVA1': {
        title: 'Left Prism Axis 1',
        subtitle: 'Prism axis accepts the following Directional or Diopter values as valid input',
        text: 'Directional: Up, Down, In, Out\n' +
            'Diopter range: 0-359 (dpt)'

    },
    'raw_data.rtPRVA2': {
        title: 'Right Prism Axis 2',
        subtitle: 'This field is only available when a direction (up, down, in, out) is used for Right Prism Axis 1'
    },
    'raw_data.ltPRVA2': {
        title: 'Left Prism Axis 2',
        subtitle: 'This field is only available when a direction (up, down, in, out) is used for Left Prism Axis 1'
    }
}

function formatDecimal(v: number, n: number) {
    return v.toFixed(n);
}

const confirmSphCyl = (name: string, value: string, opposingValue: string) => {
    if(!value) return true;
    if(value === "0") return true;
    if(!opposingValue) return true;
    if(opposingValue === "0") return true;

    const signA = Math.sign(parseFloat(value));
    const signB = Math.sign(parseFloat(opposingValue));
    if(signA !== signB) {
        window.alert(`Right and Left ${name} are using opposite power.`)
    }

}

export const GenericInput: React.FunctionComponent<OrderFieldProps> = observer(({
                                                                                    name,
                                                                                    label,
                                                                                    disabled,
                                                                                    eye,
                                                                                    isSelect,
                                                                                    options,
                                                                                    field
                                                                                }) => {
    const model = useModelContext() as OrderModel;
    const readOnly = useReadonlyContext();
    const touched = React.useRef(false);
    const [confirmedSPH, setConfirmedSphTrue, setConfirmedSphFalse] = useFlag(false)
    const [confirmedCYL, setConfirmedCylTrue, setConfirmedCylFalse] = useFlag(false)

    let theValue = model.viewModel.dig(name);
    if(theValue === undefined) {
        theValue = '';
    }

    if (theValue === model[name]) {
        const decFormat = field.increment?.split(".")[1]?.length || 0
        theValue = formatDecimal(theValue,decFormat)
    }

    const handleEyeFieldChange = React.useCallback(e => {
        model.nestedAssignment(e.target.name, e.target.value);

        if (e.target.name === "raw_data.rtSPH" || e.target.name === "raw_data.ltSPH") {
            setConfirmedSphFalse();
        }
        if (e.target.name === "raw_data.rtCYL" || e.target.name === "raw_data.ltCYL") {
            setConfirmedCylFalse();
        }
    }, [model, setConfirmedSphFalse, setConfirmedCylFalse]);

    const handleEyeFieldBlur = React.useCallback((e) => {
        const isExcludedColumn = excludedColumns.some(v => name.includes(v))
        if (eye === "rt" && !isExcludedColumn) {
            const leftName = getLeftName(name);
            if (!model.viewModel.dig(leftName)) {
                model.nestedAssignment(leftName, model.viewModel.dig(name));
            }
        }

        if(!model.viewModel.dig(name) && !isSelect && !touched.current) {
            if (field.mandatory == 1) {
                model.nestedAssignment(name, "0");
                touched.current = true;
            }
        }

        if (!e) return;

        if (e.target.name == "raw_data.rtSPH" && !confirmedSPH) {
            setConfirmedSphTrue();
            confirmSphCyl("Sphere", e.target.value, model.viewModel.raw_data.ltSPH)
        }
        if (e.target.name == "raw_data.ltSPH" && !confirmedSPH) {
            setConfirmedSphTrue();
            confirmSphCyl("Sphere", e.target.value, model.viewModel.raw_data.rtSPH)
        }
        if (e.target.name == "raw_data.rtCYL" && !confirmedCYL) {
            setConfirmedCylTrue();
            confirmSphCyl("Cylinder", e.target.value, model.viewModel.raw_data.ltCYL)
        }
        if (e.target.name == "raw_data.ltCYL" && !confirmedCYL) {
            setConfirmedCylTrue();
            confirmSphCyl("Cylinder", e.target.value, model.viewModel.raw_data.rtCYL)
        }
        model.validate();
    }, [eye, model, name, isSelect, field.mandatory, confirmedCYL, confirmedSPH, setConfirmedSphTrue, setConfirmedCylTrue]);

    const handleSelectFieldChange = React.useCallback((name, value) => {
        model.nestedAssignment(name, value);
        handleEyeFieldBlur(null);
    }, [model, handleEyeFieldBlur]);

    const errText = disabled || readOnly ? "" : model.errors[name];

    if (isSelect) {
        const theOptions = options?.map(o => ({label: o.desc ? o.desc : o.code, value: o.code})) || [];
        const theSortedOptions = theOptions.sort((a,b) => {return (a.label).localeCompare(b.label);});

        if (theValue && !theSortedOptions.find(o => o.value === theValue)) {
            theSortedOptions.push({label: theValue, value: theValue});
        }
        return (
            <SelectField
                disabled={disabled || readOnly}
                label={label}
                name={name}
                options={theSortedOptions}
                value={theValue || null}
                onChange={handleSelectFieldChange}
                error={!disabled && !!model.errors[name]}
                helperText={errText}
                clearable
                fullWidth
                hint={hintText[name]}
            />
        )
    } else {
        return (
            <InputField
                disabled={disabled || readOnly}
                label={label}
                name={name}
                value={theValue}
                onChange={handleEyeFieldChange}
                onBlur={handleEyeFieldBlur}
                error={!disabled && !!model.errors[name]}
                helperText={errText}
                fullWidth
                hint={hintText[name]}
            />
        )
    }
});