module OrderSync
  extend ActiveSupport::Concern

  class_methods do
    def submit_order!(data)
      if data["job_type"] == "F"
        process_order("local_hash_to_rxu_add_stock_hash", "add_stock_order", data)
      else
        process_order("local_hash_to_rxu_add_order_hash", "add_order", data)
      end
    end

    def submit_stock_order!(data)
      process_order("local_hash_to_rxu_add_stock_hash", "add_stock_order", data)
    end

    def rxu_hash_to_local_hash(the_hash)
      data = { "raw_data" => {} }

      the_hash.each do |key, value|
        str_key = key.to_s
        underscore_key = str_key.to_s.underscore

        case underscore_key

        when "patient_name"
          data["customer_name"] = value
          # This is only temporary until a better fix with stock orders is implemented
        when "hbox", "vbox", "fed", "dbl", "circ", "_etype", "fram", "fupc", "fcol", "_fmupc"
          if data["job_type"] == "F"
            raw_data = JSON.parse(raw_data)
            data["raw_data"][str_key] = raw_data[str_key]
          else
            data["raw_data"][str_key] = value unless data["job_type"] == "F"
          end
        when "job_type"
          data["job_type"] = get_rxu_to_local_job_type(the_hash)
          data["_fstatus"] = "D" if value == "Uncut without frame" # The API does not send _fstatus for uncut without frame jobs
        when "_fstatus"
          data["_fstatus"] = get_rxu_to_local_fstatus(the_hash)
        when "cust_job_id"
          data["customer_name"] = value
        when "trace_id"
          data["trace_id"] = get_trace(the_hash)
          data["raw_data"][str_key] = data["trace_id"] # We need to set traceid in both the raw data(for the generated fields) and top level value.
        when "date_wanted"
          if value.present?
            data["raw_data"][underscore_key] = DateTime.parse(value).strftime("%F")
          end
        when "ship_to_method"
          the_shipping_method = ShippingMethod.find_by(code: value)
          data["shipping_method_id"] = the_shipping_method&.id
        when "rem"
          data["comments"] = value
        when "unvalid_message"
          data["is_valid"] = value.blank?
          data["raw_data"][str_key] = value if value.present?
        when *top_level_dates
          data[underscore_key] = value&.to_date
        when *top_level_data
          data[underscore_key] = value

        when *addon_keys
          process_addon(underscore_key, value, data)

        when "customer_num"
          data["customer"] = find_customer(value)

        when "rt_lens_color", "rt_lens_treat"
          right_priority_single_value(underscore_key.delete_prefix("rt_"), value, data)
        when "lt_lens_color", "lt_lens_treat"
          left_subordinate_single_value(underscore_key.delete_prefix("lt_"), value, data)

        when "rt_lens_style"
          right_priority_single_value("style", find_style(value), data)
        when "lt_lens_style"
          left_subordinate_single_value("style", find_style(value), data)

        when "rt_lens_material"
          right_priority_single_value("style_material", find_style_material(value, the_hash), data)
        when "lt_lens_material"
          left_subordinate_single_value("style_material", find_style_material(value, the_hash), data)

        when "rt_class", "lt_class"
          data["raw_data"]["CLASS"] = get_rxu_to_local_class_code(value)

          # The frame information also needs to be in the top level and in raw data. We need to fix the name on frame name.
        when *frame_info
          if underscore_key == "fnam"
            data["fram"] = value
            data["raw_data"]["FRAM"] = value
          elsif underscore_key == "_fsku"
            data["raw_data"]["frame_sku"] = value.present? ? value : the_hash[:_FSKU]
            data["frame_sku"] = value.present? ? value : the_hash[:_FSKU]
          else
            data[underscore_key] = value
            data["raw_data"][str_key] = value
          end
        when "hold_status"
          data["hold_at_lab"] = value == "Y"
        when "station"
          data["job_track_station"] = value
        when "station_desc"
          data["job_track_description"] = value
        when "order_completed"
          data["is_shipped"] = value == "Y"
        else
          data["raw_data"][str_key] = value
        end
      end

      if data["style"]
        data["style_type"] = data["style"].style_type
      end

      if data["raw_data"]["date_wanted"].blank?
        data["raw_data"]["date_wanted"] = data["date_ordered"]
      end

      # Not working because orders/detail endpoint is not returning frame_sku. Not touching the code until create stock order functionality is completed
      # https://dev.azure.com/ITRxSolutions/Rx-Universe/_workitems/edit/16575
      if data["job_type"] == "F" && data["frame_sku"].present?
        set_frame_information(data)
      end

      data["raw_data"] = JSON.generate(data["raw_data"])

      data
    end

    def set_frame_information(data)
      frame = RxModule::Communicator::FrameInventory.get_frame_by_id(data["frame_sku"].to_i)
      if frame.present?
        data["fram"] = frame[:name]
        data["trace_id"] = frame[:trace]
        data["fcol"] = frame[:color]
        data["_fmupc"] = frame[:manufacturer_upc]
        data["frame_sku"] = frame[:id]
        data["fupc"] = frame[:upc]

        data["raw_data"]["FRAM"] = frame[:name]
        data["raw_data"]["traceID"] = frame[:trace]
        data["raw_data"]["FCOL"] = frame[:color]
        data["raw_data"]["_FMUPC"] = frame[:manufacturer_upc]
        data["raw_data"]["FUPC"] = frame[:upc]

        data["raw_data"]["HBOX"] = frame[:a]
        data["raw_data"]["VBOX"] = frame[:b]
        data["raw_data"]["FED"] = frame[:ed]
        data["raw_data"]["DBL"] = frame[:bridge]
        data["raw_data"]["_ETYPE"] = frame[:edge_type]

      else
        data["raw_data"]["unvalidMessage"] = "Frame not found in Rx-Inventory"
      end

      data
    rescue
      data["raw_data"]["unvalidMessage"] = "Frame not found in Rx-Inventory"
      Rails.logger.error { { msg: "Issue in fetching frame with SKU: #{data["frame_sku"]} in Rx-Inventory" } }
      data
    end

    private

    def process_order(process_method, submit_method, data)
      processed_order_data = send(process_method, data)
      Rails.logger.info { processed_order_data }
      response = RxModule::Communicator::Rxuniverse.public_send(submit_method, processed_order_data)

      unless response[:orderNumber]
        raise response.values.uniq.join("")
      end
      response[:orderNumber]
    end

    def process_trace_file(data, file)
      return unless file

      lines = file.read.lines
      radii_side = nil
      radii_count = 1
      lines.each do |line|
        key, value = line.split("=")
        value = value&.strip
        if key == "TRCFMT" || key == "ZFMT"
          _, _, _, side, _ = value.split(";")
          radii_side = side.downcase

          data_key = if key == "ZFMT"
                       "ZFmt"
                     else
                       "Fmt"
                     end

          data["#{radii_side}tTrace#{data_key}"] = value
          radii_count = 1
        elsif key == "R" || key == "Z"
          next unless radii_side

          data_key = if key == "Z"
                       "ZptsLine"
                     else
                       "RadiiLine"
                     end

          data["#{radii_side}t#{data_key}#{radii_count}"] = value
          radii_count = radii_count + 1
        end
      end
    end

    def local_hash_to_rxu_add_order_hash(the_hash)
      data = {}
      fstatus = the_hash[:_fstatus]

      raw_data = the_hash[:raw_data].is_a?(String) ? JSON.parse(the_hash[:raw_data]) : the_hash[:raw_data]

      raw_data.each do |key, value|
        if the_hash["job_type"] == "R" && key != "CLASS"
          data[key] = value unless key.start_with?("lt") || key == "rtCRIB"
          data["CRIB"] = value if key == "rtCRIB"
        elsif the_hash["job_type"] == "L" && key != "CLASS"
          data[key] = value unless key.start_with?("rt") || key == "ltCRIB"
          data["CRIB"] = value if key == "ltCRIB"
        elsif key == "date_wanted"
          data["dateWanted"] = DateTime.parse(value).strftime("%Y%m%d") unless value.blank?
        elsif key == "traceID"
          data["traceID"] = value.split(";").first(3)&.concat(raw_data.fetch_values("HBOX", "DBL"))&.join(";") if value.present?
        elsif key.to_s.include? "CRIB"
          data["CRIB"] = value if key == "rtCRIB"
        elsif ["ltAX", "rtAX"].include? key
          data[key] = value.is_a?(String) ? value.split(".")[0] : value
        elsif key == "CLASS"
          class_code = get_local_to_rxu_class_code(value)
          data["rtCLASS"] = class_code unless the_hash["job_type"] == "L"
          data["ltCLASS"] = class_code unless the_hash["job_type"] == "R"
        elsif prism_axis_fields.include? key
          if prism_axis_values.include? value
            data[key] = value[0].capitalize
          else
            data[key] = value unless value.blank?
          end
        else
          data[key] = value
        end
      end

      the_hash.each do |key, value|
        case key
        when "style_id"
          the_style = Style.find_by(id: value)
          data["rtLensStyle"] = the_style&.style unless the_hash["job_type"] == "L"
          data["ltLensStyle"] = the_style&.style unless the_hash["job_type"] == "R"
        when "style_material_id"
          the_material = StyleMaterial.find_by(id: value)
          data["rtLensMaterial"] = the_material&.material_code unless the_hash["job_type"] == "L"
          data["ltLensMaterial"] = the_material&.material_code unless the_hash["job_type"] == "R"
          data["rtLensColor"] = the_material&.tint_code unless the_material&.tint_code.blank? || the_hash["job_type"] == "L"
          data["ltLensColor"] = the_material&.tint_code unless the_material&.tint_code.blank? || the_hash["job_type"] == "R"
          data["rtLensTreat"] = the_material&.treat_code unless the_material&.treat_code.blank? || the_hash["job_type"] == "L"
          data["ltLensTreat"] = the_material&.treat_code unless the_material&.treat_code.blank? || the_hash["job_type"] == "R"
        when "shipping_method_id"
          the_shipping_method = ShippingMethod.find_by(id: value)
          data["shipMethod"] = the_shipping_method&.code
        when "addon_ids"
          value.each_with_index do |id, idx|
            the_addon = Addon.find_by(id: id)
            index = idx + 1
            data["addon#{index}"] = the_addon&.code
          end
        when "customer_id"
          data["customerNum"] = Customer.find_by(id: value)&.number
        when "customer_name"
          data["custJobID"] = value
        when "_fstatus"
          data["_FSTATUS"] = get_local_to_rxu_fstatus(value) unless fstatus == "D"
        when "job_type"
          data["jobType"] = get_local_to_rxu_job_type(value, fstatus)
          data["rtBalance"] = "Y" if value == "RB"
          data["ltBalance"] = "Y" if value == "LB"
        when "comments"
          data["REM"] = value
        when "ship_to_name"
          data["shipToName"] = change_to_full_name_format(value)
        when "style_type_id"
          # no-op, this is tied to the style
        when "raw_data"
          # no-op, this is our construct
        when "hold_at_lab"
          data["labHoldOrder"] = value ? "Y" : "N"
        when "frame_sku"
          data["_FMSKU"] = value.to_s
        when "trace_file"
          process_trace_file(data, value)
        else
          adj_key = key.camelize(:lower)
          data[adj_key] = value unless %w[custJobId traceId customerCity].any?(adj_key)
        end
      end

      data['ORIGIN'] = 'RXSITE'
      data
    end

    def local_hash_to_rxu_add_stock_hash(raw_order_info_hash)
      processed_order_data = {}
      raw_order_info_hash.each do |key, value|
        adj_key = key.camelize(:lower)
        case key
        when "comments"
          processed_order_data["REM"] = value
        when "customer_id"
          processed_order_data["customerNum"] = Customer.find_by(id: value)&.number
        when "customer_name", "reference_number"
          processed_order_data["custJobID"] = value
        when "frame_sku"
          processed_order_data["item1"] = value
          processed_order_data["quantity1"] = 1
        when "shipping_method_id"
          the_shipping_method = ShippingMethod.find_by(id: value)
          processed_order_data["shipMethod"] = the_shipping_method&.code
        when "ship_to_name"
          processed_order_data["shipToName"] = change_to_full_name_format(value)
        when "hold_at_lab"
          processed_order_data["labHoldOrder"] = value ? "Y" : "N"
        when "stock_order_items"
          value.each_with_index do |item, index|
            index = index + 1
            processed_order_data["item#{index}"] = item["item_identifier"]
            processed_order_data["quantity#{index}"] = item["quantity"]
          end
        when *frame_only_fields
          processed_order_data[adj_key] = value
        end
      end
      processed_order_data
    end

    def get_rxu_to_local_fstatus(the_hash)
      rxu_job_type = the_hash[:jobType]
      rxu_fstatus = the_hash[:_FSTATUS]
      if rxu_job_type == "Uncut with frame"
        "U"
      else
        case rxu_fstatus
        when "Supply no charge", "Supply Frame"
          "S"
        when "Frame enclosed"
          "E"
        when "Frame to follow"
          "F"
        when "Lenses only"
          "L"
        else
          "N"
        end
      end
    end

    def get_local_to_rxu_job_type(job_type, fstatus)
      return fstatus if %w[U D].any?(fstatus)
      return "F" unless %w[A E].any?(job_type)
      job_type
    end

    def get_local_to_rxu_fstatus(value)
      return value unless %w[U N].any?(value)
      value == "U" ? "L" : ""
    end

    def get_local_to_rxu_class_code(value)
      if value == "Lab choice"
        ""
      elsif value == "Force finished"
        "F"
      elsif value == "Force semi-finished"
        "S"
      end
    end

    def get_rxu_to_local_class_code(value)
      if value == ""
        "Lab choice"
      elsif value == "F"
        "Force finished"
      elsif value == "S"
        "Force semi-finished"
      end
    end

    def get_rxu_to_local_job_type(the_hash)
      if the_hash[:_BALANCE]
        the_hash[:_BALANCE] == "R" ? "RB" : "LB"
      elsif the_hash[:rtLensStyle] && the_hash[:ltLensStyle].nil?
        "R"
      elsif the_hash[:rtLensStyle].nil? && the_hash[:ltLensStyle]
        "L"
      elsif the_hash[:jobType] == "Cut and edge"
        "P"
      elsif the_hash[:jobType] == "Addon order"
        "A"
      elsif the_hash[:jobType] == "Edge only order"
        "E"
      elsif the_hash[:jobType] == "Stock order"
        "F"
      elsif %i[rtLensStyle ltLensStyle].all? { |s| the_hash.key? s } # This will check uncut jobs with both eyes. We need to move at the end so Edge Only type is checked first.
        "P"
      else
        "I"
      end
    end

    def get_trace(the_hash)
      the_hash.fetch_values(:traceID, :traceManuf, :_ETYPE, :traceEye, :traceBridge)&.join(";") unless the_hash[:traceID].blank?
    end

    def process_addon(key, value, data)
      return unless key.starts_with?("serv_desc")
      return if value.blank?
      data["addons"] ||= []
      data["addons"] << Addon.unscoped.find_or_create_by(code: value)
    end

    def change_to_full_name_format(patient_name)
      return patient_name unless patient_name.include?(",")

      names = patient_name.split(",").map(&:strip).reject(&:empty?)
      "#{names.from(1).join(" ")} #{names.first}"
    end

    def find_style_material(value, data_hash)
      find_thing = ->(key) { data_hash.find { _1.to_s.end_with?(key) && _2.present? }&.last }

      style_code = find_thing.call("LensStyle")
      tint_code = find_thing.call("LensColor")
      treat_code = find_thing.call("LensTreat")

      StyleMaterial.unscoped.find_or_create_by(style: find_style(style_code), material_code: value, tint_code: tint_code, treat_code: treat_code)
    end

    def find_style(value)
      Style.find_or_load(value)
    end

    def find_customer(value)
      Customer.unscoped.find_or_create_by(number: value)
    end


    def right_priority_single_value(key, value, data)
      data[key] = value unless value.blank? && data[key].blank?
    end

    def left_subordinate_single_value(key, value, data)
      data[key] = value if data[key].blank?
    end

    def addon_keys
      @_addon_keys ||= (0..32).map do |i|
        "%02d" % i
      end.each_with_object([]) do |i, bucket|
        bucket << "serv_auto#{i}"
        bucket << "serv_qty#{i}"
        bucket << "serv_desc#{i}"
      end
    end

    def prism_axis_fields
      %w[rtPRVA1 ltPRVA1 rtPRVA2 ltPRVA2]
    end

    def prism_axis_values
      %w[Up up U u Down down D d In in I i Out out O o]
    end

    def top_level_data
      %w[order_number customer_name customer_city po_number release_number tray ship_to_name ship_to_num ship_to_address1
        ship_to_address2 ship_to_address3 ship_to_address4 ship_to_code lens_color lens_treat] + top_level_dates
    end

    def top_level_dates
      %w[date_completed date_received date_ordered]
    end

    def frame_info
      %w[fupc fnam fcol _fmupc _fsku]
    end

    def frame_only_fields
      %w[po_number release_number shipping_method_id ship_to_num ship_to_address1 ship_to_address2 ship_to_address3 ship_to_address4 ship_to_code]
    end
  end

  def load_from_rxu_data!(data)
    assign_attributes(Order.rxu_hash_to_local_hash(data))
    assign_attributes(submitted_by: "RxUniverse") if submitted_by.blank?
    if changed?
      data["last_sync_time"] = Time.now
      data["last_sync_message"] = "success"
      data["has_synced_once"] = true
      save
    end
  end

  def sync_with_rxuniverse!
    response = RxModule::Communicator::Rxuniverse.order_details(order_number)

    if response[:jobType] == "Stock order" && !RxuVersion.instance.va?

      if StockOrder.load_from_rxuniverse(response) && persisted?
        destroy! # if this is a stock order, and we've already saved it, just delete the incorrect order
      end
    else
      load_from_rxu_data!(response)
      if errors.any?
        update(
          last_sync_time: Time.now,
          last_sync_message: JSON.generate(errors.to_h)
        )
      end
    end
  rescue => e
    if e.message.include?("not in system") && !has_synced_once
      update(
        last_sync_time: Time.now,
        last_sync_message: "Pending processing"
      )
    else
      update(
        last_sync_time: Time.now,
        last_sync_message: e.message
      )
    end
  end
end